@@ -6,7 +6,7 @@
 //
 // Copyright 2006 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright 2007, 2008, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
-// Copyright 2007-2010, 2012, 2015-2021 Albert Astals Cid <aacid@kde.org>
+// Copyright 2007-2010, 2012, 2015-2025 Albert Astals Cid <aacid@kde.org>
 // Copyright 2010 Mark Riedesel <mark@klowner.com>
 // Copyright 2011 Pino Toscano <pino@kde.org>
 // Copyright 2012 Fabio D'Urso <fabiodurso@hotmail.it>
@@ -26,20 +26,29 @@
 // Copyright 2020 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by Technische Universität Dresden
 // Copyright 2021 Georgiy Sgibnev <georgiy@sgibnev.com>. Work sponsored by lab50.net.
 // Copyright 2021 Theofilos Intzoglou <int.teo@gmail.com>
+// Copyright 2022 Alexander Sulfrian <asulfrian@zedat.fu-berlin.de>
+// Copyright 2023-2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright 2024 Pratham Gandhi <ppg.1382@gmail.com>
+// Copyright 2025 Blair Bonnett <blair.bonnett@gmail.com>
 //
 //========================================================================
 
 #ifndef FORM_H
 #define FORM_H
 
-#include "Object.h"
 #include "Annot.h"
+#include "CharTypes.h"
+#include "CryptoSignBackend.h"
+#include "Object.h"
 #include "poppler_private_export.h"
+#include "SignatureInfo.h"
 
 #include <ctime>
 
+#include <optional>
 #include <set>
 #include <vector>
+#include <functional>
 
 class GooString;
 class Array;
@@ -50,9 +59,10 @@ class Annots;
 class LinkAction;
 class GfxResources;
 class PDFDoc;
-class SignatureInfo;
 class X509CertificateInfo;
-class SignatureHandler;
+namespace CryptoSign {
+class VerificationInterface;
+}
 
 enum FormFieldType
 {
@@ -70,21 +80,6 @@ enum FormButtonType
     formButtonRadio
 };
 
-enum VariableTextQuadding
-{
-    quaddingLeftJustified,
-    quaddingCentered,
-    quaddingRightJustified
-};
-
-enum FormSignatureType
-{
-    adbe_pkcs7_sha1,
-    adbe_pkcs7_detached,
-    ETSI_CAdES_detached,
-    unknown_signature_type
-};
-
 enum FillValueType
 {
     fillValue,
@@ -115,7 +110,7 @@ public:
     // Get the field bounding rect
     void getRect(double *x1, double *y1, double *x2, double *y2) const;
 
-    unsigned getID() { return ID; }
+    unsigned getID() const { return ID; }
     void setID(unsigned int i) { ID = i; }
 
     FormField *getField() { return field; }
@@ -125,13 +120,13 @@ public:
     Ref getRef() { return ref; }
 
     void setChildNum(unsigned i) { childNum = i; }
-    unsigned getChildNum() { return childNum; }
+    unsigned getChildNum() const { return childNum; }
 
     const GooString *getPartialName() const;
     void setPartialName(const GooString &name);
     const GooString *getAlternateUiName() const;
     const GooString *getMappingName() const;
-    GooString *getFullyQualifiedName();
+    const GooString *getFullyQualifiedName();
 
     bool isModified() const;
 
@@ -140,7 +135,7 @@ public:
 
     LinkAction *getActivationAction(); // The caller should not delete the result
     std::unique_ptr<LinkAction> getAdditionalAction(Annot::FormAdditionalActionsType type);
-    bool setAdditionalAction(Annot::FormAdditionalActionsType t, const GooString &js);
+    bool setAdditionalAction(Annot::FormAdditionalActionsType t, const std::string &js);
 
     // return the unique ID corresponding to pageNum/fieldNum
     static int encodeID(unsigned pageNum, unsigned fieldNum);
@@ -148,17 +143,17 @@ public:
     static void decodeID(unsigned id, unsigned *pageNum, unsigned *fieldNum);
 
     void createWidgetAnnotation();
-    AnnotWidget *getWidgetAnnotation() const { return widget; }
-    void setWidgetAnnotation(AnnotWidget *_widget) { widget = _widget; }
+    std::shared_ptr<AnnotWidget> getWidgetAnnotation() const { return widget; }
+    void setWidgetAnnotation(std::shared_ptr<AnnotWidget> _widget) { widget = std::move(_widget); }
 
     virtual void updateWidgetAppearance() = 0;
 
-    void print(int indent = 0);
+    void print(int indent = 0) const;
 
 protected:
     FormWidget(PDFDoc *docA, Object *aobj, unsigned num, Ref aref, FormField *fieldA);
 
-    AnnotWidget *widget;
+    std::shared_ptr<AnnotWidget> widget;
     FormField *field;
     FormFieldType type;
     Object obj;
@@ -201,7 +196,7 @@ public:
 
 protected:
     FormFieldButton *parent() const;
-    GooString *onStr;
+    std::unique_ptr<GooString> onStr;
 };
 
 //------------------------------------------------------------------------
@@ -216,9 +211,9 @@ public:
     const GooString *getContent() const;
 
     // expects a UTF16BE string
-    void setContent(const GooString *new_content);
+    void setContent(std::unique_ptr<GooString> new_content);
     // sets the text inside the field appearance stream
-    void setAppearanceContent(const GooString *new_content);
+    void setAppearanceContent(std::unique_ptr<GooString> new_content);
 
     void updateWidgetAppearance() override;
 
@@ -255,7 +250,7 @@ public:
     const GooString *getExportVal(int i) const;
     // select the i-th choice
     void select(int i);
-
+    void setAppearanceChoiceContent(std::unique_ptr<GooString> new_content);
     // toggle selection of the i-th choice
     void toggle(int i);
 
@@ -264,7 +259,7 @@ public:
 
     // except a UTF16BE string
     // only work for editable combo box, set the user-entered text as the current choice
-    void setEditChoice(const GooString *new_content);
+    void setEditChoice(std::unique_ptr<GooString> new_content);
 
     const GooString *getEditChoice() const;
 
@@ -293,11 +288,24 @@ public:
     FormWidgetSignature(PDFDoc *docA, Object *dictObj, unsigned num, Ref ref, FormField *p);
     void updateWidgetAppearance() override;
 
-    FormSignatureType signatureType() const;
-    void setSignatureType(FormSignatureType fst);
+    CryptoSign::SignatureType signatureType() const;
+    void setSignatureType(CryptoSign::SignatureType fst);
 
     // Use -1 for now as validationTime
-    SignatureInfo *validateSignature(bool doVerifyCert, bool forceRevalidation, time_t validationTime, bool ocspRevocationCheck, bool enableAIA);
+    // ocspRevocation and aiafetch might happen async in the Background
+    // doneCallback will be invoked once there is a result
+    // Note: Validation callback will likely happen from an auxillary
+    // thread and it is the caller of this method who is responsible
+    // for moving back to the main thread
+    // For synchronous code, don't provide validation callback
+    // and just call validateSignatureResult afterwards
+    // The returned SignatureInfo from this method does
+    // not have validated the certificate.
+    SignatureInfo *validateSignatureAsync(bool doVerifyCert, bool forceRevalidation, time_t validationTime, bool ocspRevocationCheck, bool enableAIA, const std::function<void()> &doneCallback);
+
+    /// Waits, if needed, on validation callback and
+    /// returns a signatureinfo with validated certificates
+    CertificateValidationStatus validateSignatureResult();
 
     // returns a list with the boundaries of the signed ranges
     // the elements of the list are of type Goffset
@@ -310,21 +318,28 @@ public:
     // field "ByteRange" in the dictionary "V".
     // Arguments reason and location are UTF-16 big endian strings with BOM. An empty string and nullptr are acceptable too.
     // Returns success.
-    bool signDocument(const char *filename, const char *certNickname, const char *digestName, const char *password, const GooString *reason = nullptr, const GooString *location = nullptr);
+    std::optional<CryptoSign::SigningErrorMessage> signDocument(const std::string &filename, const std::string &certNickname, const std::string &password, const GooString *reason = nullptr, const GooString *location = nullptr,
+                                                                const std::optional<GooString> &ownerPassword = {}, const std::optional<GooString> &userPassword = {});
 
-    // checks the length encoding of the signature and returns the hex encoded signature
-    // if the check passed (and the checked file size as output parameter in checkedFileSize)
-    // otherwise a nullptr is returned
-    GooString *getCheckedSignature(Goffset *checkedFileSize);
+    // Same as above but adds text, font color, etc.
+    std::optional<CryptoSign::SigningErrorMessage> signDocumentWithAppearance(const std::string &filename, const std::string &certNickname, const std::string &password, const GooString *reason = nullptr, const GooString *location = nullptr,
+                                                                              const std::optional<GooString> &ownerPassword = {}, const std::optional<GooString> &userPassword = {}, const GooString &signatureText = {},
+                                                                              const GooString &signatureTextLeft = {}, double fontSize = {}, double leftFontSize = {}, std::unique_ptr<AnnotColor> &&fontColor = {}, double borderWidth = {},
+                                                                              std::unique_ptr<AnnotColor> &&borderColor = {}, std::unique_ptr<AnnotColor> &&backgroundColor = {});
 
-    const GooString *getSignature() const;
+    // checks the length encoding of the signature and returns the signature
+    // and the length that the signature is supposed to cover.
+    // if invalid, a empty optional and 0 is returned.
+    std::pair<std::optional<std::vector<unsigned char>>, int64_t> getCheckedSignature();
+
+    const std::vector<unsigned char> &getSignature() const;
 
 private:
-    bool createSignature(Object &vObj, Ref vRef, const GooString &name, const GooString *signature, const GooString *reason = nullptr, const GooString *location = nullptr);
-    bool getObjectStartEnd(GooString *filename, int objNum, Goffset *objStart, Goffset *objEnd);
-    bool updateOffsets(FILE *f, Goffset objStart, Goffset objEnd, Goffset *sigStart, Goffset *sigEnd, Goffset *fileSize);
+    bool createSignature(Object &vObj, Ref vRef, const GooString &name, int placeholderLength, const GooString *reason, const GooString *location, CryptoSign::SignatureType signatureType);
+    static bool getObjectStartEnd(const GooString &filename, int objNum, Goffset *objStart, Goffset *objEnd, const std::optional<GooString> &ownerPassword, const std::optional<GooString> &userPassword);
+    static bool updateOffsets(FILE *f, Goffset objStart, Goffset objEnd, Goffset *sigStart, Goffset *sigEnd, Goffset *fileSize);
 
-    bool updateSignature(FILE *f, Goffset sigStart, Goffset sigEnd, const GooString *signature);
+    static bool updateSignature(FILE *f, Goffset sigStart, Goffset sigEnd, const std::vector<unsigned char> &signature);
 };
 
 //------------------------------------------------------------------------
@@ -334,10 +349,10 @@ private:
 // only interact with FormWidgets.
 //------------------------------------------------------------------------
 
-class FormField
+class POPPLER_PRIVATE_EXPORT FormField
 {
 public:
-    FormField(PDFDoc *docA, Object &&aobj, const Ref aref, FormField *parent, std::set<int> *usedParents, FormFieldType t = formUndef);
+    FormField(PDFDoc *docA, Object &&aobj, Ref aref, FormField *parent, std::set<int> *usedParents, FormFieldType t = formUndef);
 
     virtual ~FormField();
 
@@ -351,21 +366,23 @@ public:
     void setStandAlone(bool value) { standAlone = value; }
     bool isStandAlone() const { return standAlone; }
 
-    GooString *getDefaultAppearance() const { return defaultAppearance; }
+    GooString *getDefaultAppearance() const { return defaultAppearance.get(); }
+    void setDefaultAppearance(const std::string &appearance);
+
     bool hasTextQuadding() const { return hasQuadding; }
     VariableTextQuadding getTextQuadding() const { return quadding; }
 
-    const GooString *getPartialName() const { return partialName; }
+    const GooString *getPartialName() const { return partialName.get(); }
     void setPartialName(const GooString &name);
-    const GooString *getAlternateUiName() const { return alternateUiName; }
-    const GooString *getMappingName() const { return mappingName; }
-    GooString *getFullyQualifiedName();
+    const GooString *getAlternateUiName() const { return alternateUiName.get(); }
+    const GooString *getMappingName() const { return mappingName.get(); }
+    const GooString *getFullyQualifiedName() const;
 
     FormWidget *findWidgetByRef(Ref aref);
-    int getNumWidgets() const { return terminal ? numChildren : 0; }
-    FormWidget *getWidget(int i) const { return terminal ? widgets[i] : nullptr; }
-    int getNumChildren() const { return !terminal ? numChildren : 0; }
-    FormField *getChildren(int i) const { return children[i]; }
+    int getNumWidgets() const { return terminal ? widgets.size() : 0; }
+    FormWidget *getWidget(int i) const { return terminal ? widgets[i].get() : nullptr; }
+    int getNumChildren() const { return !terminal ? children.size() : 0; }
+    FormField *getChildren(int i) const { return !terminal ? children[i].get() : nullptr; }
 
     // only implemented in FormFieldButton
     virtual void fillChildrenSiblingsID();
@@ -379,6 +396,8 @@ public:
     FormField *findFieldByRef(Ref aref);
     FormField *findFieldByFullyQualifiedName(const std::string &name);
 
+    bool getNoExport() const { return noExport; }
+
 protected:
     void _createWidget(Object *obj, Ref aref);
     void createChildren(std::set<int> *usedParents);
@@ -391,19 +410,19 @@ protected:
     Object obj;
     PDFDoc *doc;
     XRef *xref;
-    FormField **children;
+    std::vector<std::unique_ptr<FormField>> children;
     FormField *parent;
-    int numChildren;
-    FormWidget **widgets;
+    std::vector<std::unique_ptr<FormWidget>> widgets;
     bool readOnly;
+    bool noExport;
 
-    GooString *partialName; // T field
-    GooString *alternateUiName; // TU field
-    GooString *mappingName; // TM field
-    GooString *fullyQualifiedName;
+    std::unique_ptr<GooString> partialName; // T field
+    std::unique_ptr<GooString> alternateUiName; // TU field
+    std::unique_ptr<GooString> mappingName; // TM field
+    mutable std::unique_ptr<GooString> fullyQualifiedName;
 
     // Variable Text
-    GooString *defaultAppearance;
+    std::unique_ptr<GooString> defaultAppearance;
     bool hasQuadding;
     VariableTextQuadding quadding;
 
@@ -411,7 +430,7 @@ protected:
     bool standAlone;
 
 private:
-    FormField() { }
+    FormField() = default;
 };
 
 //------------------------------------------------------------------------
@@ -421,7 +440,7 @@ private:
 class FormFieldButton : public FormField
 {
 public:
-    FormFieldButton(PDFDoc *docA, Object &&dict, const Ref ref, FormField *parent, std::set<int> *usedParents);
+    FormFieldButton(PDFDoc *docA, Object &&dict, Ref ref, FormField *parent, std::set<int> *usedParents);
 
     FormButtonType getButtonType() const { return btype; }
 
@@ -441,7 +460,7 @@ public:
 
     // For radio buttons, return the fields of the other radio buttons in the same group
     FormFieldButton *getSibling(int i) const { return siblings[i]; }
-    int getNumSiblings() const { return numSiblings; }
+    int getNumSiblings() const { return int(siblings.size()); }
 
     void print(int indent) override;
     void reset(const std::vector<std::string> &excludedFields) override;
@@ -451,10 +470,8 @@ public:
 protected:
     void updateState(const char *state);
 
-    FormFieldButton **siblings; // IDs of dependent buttons (each button of a radio field has all the others buttons
-                                // of the same field in this array)
-    int numSiblings;
-
+    std::vector<FormFieldButton *> siblings; // IDs of dependent buttons (each button of a radio field has all the others buttons
+                                             // of the same field in this array)
     FormButtonType btype;
     int size;
     int active_child; // only used for combo box
@@ -470,12 +487,12 @@ protected:
 class FormFieldText : public FormField
 {
 public:
-    FormFieldText(PDFDoc *docA, Object &&dictObj, const Ref ref, FormField *parent, std::set<int> *usedParents);
+    FormFieldText(PDFDoc *docA, Object &&dictObj, Ref ref, FormField *parent, std::set<int> *usedParents);
 
-    const GooString *getContent() const { return content; }
-    const GooString *getAppearanceContent() const { return internalContent ? internalContent : content; }
-    void setContentCopy(const GooString *new_content);
-    void setAppearanceContentCopy(const GooString *new_content);
+    const GooString *getContent() const { return content.get(); }
+    const GooString *getAppearanceContent() const { return internalContent ? internalContent.get() : content.get(); }
+    void setContent(std::unique_ptr<GooString> new_content);
+    void setAppearanceContent(std::unique_ptr<GooString> new_content);
     ~FormFieldText() override;
 
     bool isMultiline() const { return multiline; }
@@ -496,15 +513,15 @@ public:
     void print(int indent) override;
     void reset(const std::vector<std::string> &excludedFields) override;
 
-    static int tokenizeDA(const std::string &daString, std::vector<std::string> *daToks, const char *searchTok);
+    static std::optional<size_t> tokenizeDA(const std::string &daString, std::vector<std::string> *daToks, const char *searchTok);
 
 protected:
-    int parseDA(std::vector<std::string> *daToks);
+    std::optional<size_t> parseDA(std::vector<std::string> *daToks) const;
     void fillContent(FillValueType fillType);
 
-    GooString *content;
-    GooString *internalContent;
-    GooString *defaultContent;
+    std::unique_ptr<GooString> content;
+    std::unique_ptr<GooString> internalContent;
+    std::unique_ptr<GooString> defaultContent;
     bool multiline;
     bool password;
     bool fileSelect;
@@ -522,15 +539,18 @@ protected:
 class FormFieldChoice : public FormField
 {
 public:
-    FormFieldChoice(PDFDoc *docA, Object &&aobj, const Ref ref, FormField *parent, std::set<int> *usedParents);
+    FormFieldChoice(PDFDoc *docA, Object &&aobj, Ref ref, FormField *parent, std::set<int> *usedParents);
 
     ~FormFieldChoice() override;
 
     int getNumChoices() const { return numChoices; }
-    const GooString *getChoice(int i) const { return choices ? choices[i].optionName : nullptr; }
-    const GooString *getExportVal(int i) const { return choices ? choices[i].exportVal : nullptr; }
+    const GooString *getChoice(int i) const { return choices ? choices[i].optionName.get() : nullptr; }
+    const GooString *getExportVal(int i) const { return choices ? choices[i].exportVal.get() : nullptr; }
     // For multi-select choices it returns the first one
     const GooString *getSelectedChoice() const;
+    const GooString *getAppearanceSelectedChoice() const { return appearanceSelectedChoice ? appearanceSelectedChoice.get() : getSelectedChoice(); }
+
+    void setAppearanceChoiceContent(std::unique_ptr<GooString> new_content);
 
     // select the i-th choice
     void select(int i);
@@ -542,7 +562,7 @@ public:
     void deselectAll();
 
     // only work for editable combo box, set the user-entered text as the current choice
-    void setEditChoice(const GooString *new_content);
+    void setEditChoice(std::unique_ptr<GooString> new_content);
 
     const GooString *getEditChoice() const;
 
@@ -575,15 +595,16 @@ protected:
 
     struct ChoiceOpt
     {
-        GooString *exportVal; // the export value ("internal" name)
-        GooString *optionName; // displayed name
-        bool selected; // if this choice is selected
+        std::unique_ptr<GooString> exportVal; // the export value ("internal" name)
+        std::unique_ptr<GooString> optionName; // displayed name
+        bool selected = false; // if this choice is selected
     };
 
     int numChoices;
     ChoiceOpt *choices;
     bool *defaultChoices;
-    GooString *editedChoice;
+    std::unique_ptr<GooString> editedChoice;
+    std::unique_ptr<GooString> appearanceSelectedChoice;
     int topIdx; // TI
 };
 
@@ -594,26 +615,28 @@ protected:
 class POPPLER_PRIVATE_EXPORT FormFieldSignature : public FormField
 {
 public:
-    FormFieldSignature(PDFDoc *docA, Object &&dict, const Ref ref, FormField *parent, std::set<int> *usedParents);
+    FormFieldSignature(PDFDoc *docA, Object &&dict, Ref ref, FormField *parent, std::set<int> *usedParents);
 
     // Use -1 for now as validationTime
-    SignatureInfo *validateSignature(bool doVerifyCert, bool forceRevalidation, time_t validationTime, bool ocspRevocationCheck, bool enableAIA);
+    SignatureInfo *validateSignatureAsync(bool doVerifyCert, bool forceRevalidation, time_t validationTime, bool ocspRevocationCheck, bool enableAIA, const std::function<void()> &doneCallback);
+
+    CertificateValidationStatus validateSignatureResult();
 
     // returns a list with the boundaries of the signed ranges
     // the elements of the list are of type Goffset
     std::vector<Goffset> getSignedRangeBounds() const;
 
-    // checks the length encoding of the signature and returns the hex encoded signature
-    // if the check passed (and the checked file size as output parameter in checkedFileSize)
-    // otherwise a nullptr is returned
-    GooString *getCheckedSignature(Goffset *checkedFileSize);
+    // checks the length encoding of the signature and returns the signature
+    // and the end of data the signature covers
+    // otherwise a nullopt and 0 is returned
+    std::pair<std::optional<std::vector<unsigned char>>, int64_t> getCheckedSignature();
 
     ~FormFieldSignature() override;
     Object *getByteRange() { return &byte_range; }
-    const GooString *getSignature() const { return signature; }
-    void setSignature(const GooString &sig);
-    FormSignatureType getSignatureType() const { return signature_type; }
-    void setSignatureType(FormSignatureType t) { signature_type = t; }
+    const std::vector<unsigned char> &getSignature() const { return signature; }
+    void setSignature(std::vector<unsigned char> &&sig);
+    CryptoSign::SignatureType getSignatureType() const { return signature_type; }
+    void setSignatureType(CryptoSign::SignatureType t) { signature_type = t; }
 
     const GooString &getCustomAppearanceContent() const;
     void setCustomAppearanceContent(const GooString &s);
@@ -624,22 +647,28 @@ public:
     double getCustomAppearanceLeftFontSize() const;
     void setCustomAppearanceLeftFontSize(double size);
 
-    void setCertificateInfo(std::unique_ptr<X509CertificateInfo> &);
+    // Background image (ref to an object of type XObject). Invalid ref if not required.
+    Ref getImageResource() const;
+    void setImageResource(Ref imageResourceA);
+
+    void setCertificateInfo(std::unique_ptr<X509CertificateInfo> &certInfo);
 
     FormWidget *getCreateWidget();
 
 private:
     void parseInfo();
-    void hashSignedDataBlock(SignatureHandler *handler, Goffset block_len);
+    void hashSignedDataBlock(CryptoSign::VerificationInterface *handler, Goffset block_len);
 
-    FormSignatureType signature_type;
+    CryptoSign::SignatureType signature_type = CryptoSign::SignatureType::unsigned_signature_field;
     Object byte_range;
-    GooString *signature;
+    std::vector<unsigned char> signature;
     SignatureInfo *signature_info;
     GooString customAppearanceContent;
     GooString customAppearanceLeftContent;
     double customAppearanceLeftFontSize = 20;
+    Ref imageResource = Ref::INVALID();
     std::unique_ptr<X509CertificateInfo> certificate_info;
+    std::unique_ptr<CryptoSign::VerificationInterface> signature_handler;
 
     void print(int indent) override;
 };
@@ -653,7 +682,7 @@ private:
 class POPPLER_PRIVATE_EXPORT Form
 {
 public:
-    Form(PDFDoc *docA, Object *acroForm);
+    explicit Form(PDFDoc *doc);
 
     ~Form();
 
@@ -666,13 +695,41 @@ public:
     /* Creates a new Field of the type specified in obj's dict.
        used in Form::Form , FormField::FormField and
        Page::loadStandaloneFields */
-    static FormField *createFieldFromDict(Object &&obj, PDFDoc *docA, const Ref aref, FormField *parent, std::set<int> *usedParents);
+    static std::unique_ptr<FormField> createFieldFromDict(Object &&obj, PDFDoc *docA, Ref aref, FormField *parent, std::set<int> *usedParents);
+
+    // Finds in the default resources dictionary a font named popplerfontXXX that
+    // has the given fontFamily and fontStyle. This makes us relatively sure that we added that font ourselves
+    std::string findFontInDefaultResources(const std::string &fontFamily, const std::string &fontStyle) const;
+
+    // Finds in the default resources a font that is suitable to create a signature annotation.
+    // If none is found then it is added to the default resources.
+    std::string findPdfFontNameToUseForSigning();
+
+    struct AddFontResult
+    {
+        std::string fontName;
+        Ref ref;
+    };
+
+    // Finds in the system a font name matching the given fontFamily and fontStyle
+    // And adds it to the default resources dictionary, font name there will be popplerfontXXX except if forceName is true,
+    // in that case the font name will be fontFamily + " " + fontStyle (if fontStyle is empty just fontFamily)
+    AddFontResult addFontToDefaultResources(const std::string &fontFamily, const std::string &fontStyle, bool forceName = false);
+
+    // Finds in the default resources dictionary a font named popplerfontXXX that
+    // emulates fontToEmulate and can draw the given char
+    std::string getFallbackFontForChar(Unicode uChar, const GfxFont &fontToEmulate) const;
+
+    // Makes sure the default resources has fonts to draw all the given chars and as close as possible to the given pdfFontNameToEmulate
+    // If needed adds fonts to the default resources dictionary, font names will be popplerfontXXX
+    // If fieldResources is not nullptr, it is used instead of the to query the font to emulate instead of the default resources
+    // Returns a list of all the added fonts (if any)
+    std::vector<AddFontResult> ensureFontsForAllCharacters(const GooString *unicodeText, const std::string &pdfFontNameToEmulate, GfxResources *fieldResources = nullptr);
 
-    Object *getObj() const { return acroForm; }
     bool getNeedAppearances() const { return needAppearances; }
-    int getNumFields() const { return numFields; }
-    FormField *getRootField(int i) const { return rootFields[i]; }
-    const GooString *getDefaultAppearance() const { return defaultAppearance; }
+    int getNumFields() const { return rootFields.size(); }
+    FormField *getRootField(int i) const { return rootFields[i].get(); }
+    const GooString *getDefaultAppearance() const { return defaultAppearance.get(); }
     VariableTextQuadding getTextQuadding() const { return quadding; }
     GfxResources *getDefaultResources() const { return defaultResources; }
     Object *getDefaultResourcesObj() { return &resDict; }
@@ -680,6 +737,7 @@ public:
     FormWidget *findWidgetByRef(Ref aref);
     FormField *findFieldByRef(Ref aref) const;
     FormField *findFieldByFullyQualifiedName(const std::string &name) const;
+    FormField *findFieldByFullyQualifiedNameOrRef(const std::string &field) const;
 
     void postWidgetsLoad();
 
@@ -688,19 +746,23 @@ public:
     void reset(const std::vector<std::string> &fields, bool excludeFields);
 
 private:
-    FormField **rootFields;
-    int numFields;
-    int size;
-    PDFDoc *doc;
-    XRef *xref;
-    Object *acroForm;
+    // Finds in the system a font name matching the given fontFamily and fontStyle
+    // And adds it to the default resources dictionary, font name there will be popplerfontXXX except if forceName is true,
+    // in that case the font name will be fontFamily + " " + fontStyle (if fontStyle is empty just fontFamily)
+    // if fileSubstitutedIn is true,
+    AddFontResult addFontToDefaultResources(const std::string &filepath, int faceIndex, const std::string &fontFamily, const std::string &fontStyle, bool fontSubstitutedIn, bool forceName);
+
+    AddFontResult doGetAddFontToDefaultResources(Unicode uChar, const GfxFont &fontToEmulate);
+
+    std::vector<std::unique_ptr<FormField>> rootFields;
+    PDFDoc *const doc;
     bool needAppearances;
     GfxResources *defaultResources;
     Object resDict;
     std::vector<Ref> calculateOrder;
 
     // Variable Text
-    GooString *defaultAppearance;
+    std::unique_ptr<GooString> defaultAppearance;
     VariableTextQuadding quadding;
 };
 
@@ -717,14 +779,12 @@ public:
     FormPageWidgets(const FormPageWidgets &) = delete;
     FormPageWidgets &operator=(const FormPageWidgets &) = delete;
 
-    int getNumWidgets() const { return numWidgets; }
+    int getNumWidgets() const { return widgets.size(); }
     FormWidget *getWidget(int i) const { return widgets[i]; }
-    void addWidgets(const std::vector<FormField *> &addedWidgets, unsigned int page);
+    void addWidgets(const std::vector<std::unique_ptr<FormField>> &addedWidgets, unsigned int page);
 
 private:
-    FormWidget **widgets;
-    int numWidgets;
-    int size;
+    std::vector<FormWidget *> widgets;
 };
 
 #endif
