@@ -17,7 +17,7 @@
 // Copyright (C) 2006, 2007 Jeff Muizelaar <jeff@infidigm.net>
 // Copyright (C) 2006 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009 Koji Otani <sho@bbr.jp>
-// Copyright (C) 2009-2011, 2013, 2016-2022, 2024, 2025 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2009-2011, 2013, 2016-2022, 2024-2026 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Christian Feuersänger <cfeuersaenger@googlemail.com>
 // Copyright (C) 2011 Andrea Canciani <ranma42@gmail.com>
 // Copyright (C) 2011-2014, 2016, 2020 Thomas Freitag <Thomas.Freitag@alfa.de>
@@ -64,7 +64,7 @@ class GfxSeparationColorSpace;
 class Matrix
 {
 public:
-    double m[6];
+    std::array<double, 6> m;
 
     void init(double xx, double yx, double xy, double yy, double x0, double y0)
     {
@@ -112,11 +112,11 @@ enum GfxBlendMode
 //------------------------------------------------------------------------
 
 // 16.16 fixed point color component
-typedef int GfxColorComp;
+using GfxColorComp = int;
 
-#define gfxColorComp1 0x10000
+static constexpr GfxColorComp gfxColorComp1 = 0x10000;
 
-static inline GfxColorComp dblToCol(double x)
+static constexpr GfxColorComp dblToCol(double x)
 {
     return (GfxColorComp)(x * gfxColorComp1);
 }
@@ -133,7 +133,7 @@ static inline unsigned char dblToByte(double x)
 
 static inline double byteToDbl(unsigned char x)
 {
-    return (double)x / (double)255.0;
+    return (double)x / 255.0;
 }
 
 static inline GfxColorComp byteToCol(unsigned char x)
@@ -176,7 +176,7 @@ static inline void clearGfxColor(GfxColor *gfxColor)
 // GfxGray
 //------------------------------------------------------------------------
 
-typedef GfxColorComp GfxGray;
+using GfxGray = GfxColorComp;
 
 //------------------------------------------------------------------------
 // GfxRGB
@@ -223,7 +223,7 @@ enum GfxColorSpaceMode
 // This shall hold a cmsHPROFILE handle.
 // Only use the make_GfxLCMSProfilePtr function to construct this pointer,
 // to ensure that the resources are properly released after usage.
-typedef std::shared_ptr<void> GfxLCMSProfilePtr;
+using GfxLCMSProfilePtr = std::shared_ptr<void>;
 
 #if USE_CMS
 GfxLCMSProfilePtr POPPLER_PRIVATE_EXPORT make_GfxLCMSProfilePtr(void *profile);
@@ -444,7 +444,7 @@ public:
     int getNComps() const override { return 4; }
 
     // GfxDeviceRGBAColorSpace-specific access
-    void getARGBPremultipliedLine(unsigned char *in, unsigned int *out, int length);
+    static void getARGBPremultipliedLine(unsigned char *in, unsigned int *out, int length);
 
 private:
 };
@@ -512,7 +512,7 @@ public:
     void getCMYK(const GfxColor *color, GfxCMYK *cmyk) const override;
     void getDeviceN(const GfxColor *color, GfxColor *deviceN) const override;
     void getRGBLine(unsigned char *in, unsigned int *out, int length) override;
-    void getRGBLine(unsigned char *, unsigned char *out, int length) override;
+    void getRGBLine(unsigned char *in, unsigned char *out, int length) override;
     void getRGBXLine(unsigned char *in, unsigned char *out, int length) override;
     void getCMYKLine(unsigned char *in, unsigned char *out, int length) override;
     void getDeviceNLine(unsigned char *in, unsigned char *out, int length) override;
@@ -567,7 +567,7 @@ private:
     double whiteX, whiteY, whiteZ; // white point
     double blackX, blackY, blackZ; // black point
     double aMin, aMax, bMin, bMax; // range for the a and b components
-    void getXYZ(const GfxColor *color, double *pX, double *pY, double *pZ) const;
+    static void getXYZ(const GfxColor *color, double *pX, double *pY, double *pZ);
 #if USE_CMS
     std::shared_ptr<GfxColorTransform> transform;
 #endif
@@ -720,7 +720,8 @@ public:
     GfxColorSpace *getAlt() { return alt.get(); }
     const Function *getFunc() const { return func.get(); }
 
-    GfxSeparationColorSpace(std::unique_ptr<GooString> &&nameA, std::unique_ptr<GfxColorSpace> &&altA, std::unique_ptr<Function> funcA, bool nonMarkingA, unsigned int overprintMaskA, const std::vector<int> &mappingA, PrivateTag = {});
+    GfxSeparationColorSpace(std::unique_ptr<GooString> &&nameA, std::unique_ptr<GfxColorSpace> &&altA, std::unique_ptr<Function> funcA, bool nonMarkingA, unsigned int overprintMaskA, const std::vector<int> &mappingA,
+                            PrivateTag /*unused*/ = {});
 
 private:
     const std::unique_ptr<GooString> name; // colorant name
@@ -766,7 +767,7 @@ public:
     const Function *getTintTransformFunc() const { return func.get(); }
 
     GfxDeviceNColorSpace(int nCompsA, const std::vector<std::string> &namesA, std::unique_ptr<GfxColorSpace> &&alt, std::unique_ptr<Function> func, std::vector<std::unique_ptr<GfxSeparationColorSpace>> &&sepsCSA,
-                         const std::vector<int> &mappingA, bool nonMarkingA, unsigned int overprintMaskA, PrivateTag = {});
+                         const std::vector<int> &mappingA, bool nonMarkingA, unsigned int overprintMaskA, PrivateTag /*unused*/ = {});
 
 private:
     const int nComps; // number of components
@@ -1128,14 +1129,14 @@ public:
     /**
      * @precondition isParameterized() == false
      */
-    void getTriangle(int i, double *x0, double *y0, GfxColor *color0, double *x1, double *y1, GfxColor *color1, double *x2, double *y2, GfxColor *color2);
+    void getTriangle(int i, double *x0, double *y0, GfxColor *color0, double *x1, double *y1, GfxColor *color1, double *x2, double *y2, GfxColor *color2) const;
 
     /**
      * Variant for functions.
      *
      * @precondition isParameterized() == true
      */
-    void getTriangle(int i, double *x0, double *y0, double *color0, double *x1, double *y1, double *color1, double *x2, double *y2, double *color2);
+    void getTriangle(int i, double *x0, double *y0, double *color0, double *x1, double *y1, double *color1, double *x2, double *y2, double *color2) const;
 
     void getParameterizedColor(double t, GfxColor *color) const;
 
@@ -1512,12 +1513,12 @@ public:
 
     private:
         GfxPath *path;
-        int subPathOff;
+        int subPathOff = 0;
 
-        int coordOff;
-        int numCoords;
+        int coordOff = 0;
+        int numCoords = 0;
 
-        GfxSubpath *curSubPath;
+        GfxSubpath *curSubPath = nullptr;
     };
 
     enum LineJoinStyle
@@ -1550,8 +1551,8 @@ public:
     // Accessors.
     double getHDPI() const { return hDPI; }
     double getVDPI() const { return vDPI; }
-    const double *getCTM() const { return ctm; }
-    void getCTM(Matrix *m) const { memcpy(m->m, ctm, sizeof m->m); }
+    const std::array<double, 6> &getCTM() const { return ctm; }
+    void getCTM(Matrix *m) const { m->m = ctm; }
     double getX1() const { return px1; }
     double getY1() const { return py1; }
     double getX2() const { return px2; }
@@ -1595,7 +1596,7 @@ public:
     bool getTextKnockout() const { return textKnockout; }
     const std::shared_ptr<GfxFont> &getFont() const { return font; }
     double getFontSize() const { return fontSize; }
-    const double *getTextMat() const { return textMat; }
+    const std::array<double, 6> &getTextMat() const { return textMat; }
     double getCharSpace() const { return charSpace; }
     double getWordSpace() const { return wordSpace; }
     double getHorizScaling() const { return horizScaling; }
@@ -1604,7 +1605,6 @@ public:
     int getRender() const { return render; }
     const char *getRenderingIntent() const { return renderingIntent; }
     const GfxPath *getPath() const { return path; }
-    void setPath(GfxPath *pathA);
     double getCurX() const { return curX; }
     double getCurY() const { return curY; }
     double getCurTextX() const { return curTextX; }
@@ -1670,8 +1670,8 @@ public:
     void setLineWidth(double width) { lineWidth = width; }
     void setLineDash(std::vector<double> &&dash, double start);
     void setFlatness(int flatness1) { flatness = flatness1; }
-    void setLineJoin(int lineJoin1) { lineJoin = static_cast<LineJoinStyle>(lineJoin1); }
-    void setLineCap(int lineCap1) { lineCap = static_cast<LineCapStyle>(lineCap1); }
+    void setLineJoin(LineJoinStyle lineJoin1) { lineJoin = lineJoin1; }
+    void setLineCap(LineCapStyle lineCap1) { lineCap = lineCap1; }
     void setMiterLimit(double limit) { miterLimit = limit; }
     void setStrokeAdjust(bool sa) { strokeAdjust = sa; }
     void setAlphaIsShape(bool ais) { alphaIsShape = ais; }
@@ -1699,7 +1699,7 @@ public:
     GfxLCMSProfilePtr getDisplayProfile() { return localDisplayProfile; }
     void setXYZ2DisplayTransforms(std::shared_ptr<GfxXYZ2DisplayTransforms> transforms);
     std::shared_ptr<GfxColorTransform> getXYZ2DisplayTransform();
-    int getCmsRenderingIntent();
+    int getCmsRenderingIntent() const;
     static GfxLCMSProfilePtr sRGBProfile;
 #endif
 
@@ -1767,13 +1767,13 @@ public:
     bool isParentState(GfxState *state) { return saved == state || (saved && saved->isParentState(state)); }
 
     // Misc
-    bool parseBlendMode(Object *obj, GfxBlendMode *mode);
+    static bool parseBlendMode(Object *obj, GfxBlendMode *mode);
 
     std::unique_ptr<ReusablePathIterator> getReusablePath() { return std::make_unique<ReusablePathIterator>(path); }
 
 private:
     double hDPI, vDPI; // resolution
-    double ctm[6]; // coord transform matrix
+    std::array<double, 6> ctm; // coord transform matrix
     double px1, py1, px2, py2; // page corners (user coords)
     double pageWidth, pageHeight; // page size (pixels)
     int rotate; // page rotation angle
@@ -1808,7 +1808,7 @@ private:
 
     std::shared_ptr<GfxFont> font; // font
     double fontSize; // font size
-    double textMat[6]; // text matrix
+    std::array<double, 6> textMat; // text matrix
     double charSpace; // character spacing
     double wordSpace; // word spacing
     double horizScaling; // horizontal scaling
